@@ -33,6 +33,8 @@
 #include <geos/geom/PrecisionModel.h>
 #include <geos/util/IllegalArgumentException.h>
 
+#include <ryu/ryu.h>
+
 #include <algorithm> // for min
 #include <typeinfo>
 #include <cstdio> // should avoid this
@@ -42,7 +44,7 @@
 #include <cmath>
 #include <iomanip>
 
-using namespace std;
+
 using namespace geos::geom;
 
 namespace geos {
@@ -61,7 +63,7 @@ WKTWriter::WKTWriter():
 
 /* public */
 void
-WKTWriter::setOutputDimension(int dims)
+WKTWriter::setOutputDimension(uint8_t dims)
 {
     if(dims < 2 || dims > 3) {
         throw util::IllegalArgumentException("WKT output dimension must be 2 or 3");
@@ -70,10 +72,10 @@ WKTWriter::setOutputDimension(int dims)
 }
 
 /*static*/
-string
+std::string
 WKTWriter::toLineString(const CoordinateSequence& seq)
 {
-    stringstream buf(ios_base::in | ios_base::out);
+    std::stringstream buf(std::ios_base::in | std::ios_base::out);
     buf << "LINESTRING ";
     auto npts = seq.size();
     if(npts == 0) {
@@ -81,7 +83,7 @@ WKTWriter::toLineString(const CoordinateSequence& seq)
     }
     else {
         buf << "(";
-        for(size_t i = 0; i < npts; ++i) {
+        for(std::size_t i = 0; i < npts; ++i) {
             if(i) {
                 buf << ", ";
             }
@@ -97,10 +99,10 @@ WKTWriter::toLineString(const CoordinateSequence& seq)
 }
 
 /*static*/
-string
+std::string
 WKTWriter::toLineString(const Coordinate& p0, const Coordinate& p1)
 {
-    stringstream ret(ios_base::in | ios_base::out);
+    std::stringstream ret(std::ios_base::in | std::ios_base::out);
     ret << "LINESTRING (" << p0.x << " " << p0.y;
 #if PRINT_Z
     ret << " " << p0.z;
@@ -115,10 +117,10 @@ WKTWriter::toLineString(const Coordinate& p0, const Coordinate& p1)
 }
 
 /*static*/
-string
+std::string
 WKTWriter::toPoint(const Coordinate& p0)
 {
-    stringstream ret(ios_base::in | ios_base::out);
+    std::stringstream ret(std::ios_base::in | std::ios_base::out);
     ret << "POINT (";
 #if PRINT_Z
     ret << p0.x << " " << p0.y  << " " << p0.z << " )";
@@ -143,12 +145,12 @@ WKTWriter::setTrim(bool p0)
     trim = p0;
 }
 
-string
+std::string
 WKTWriter::write(const Geometry* geometry)
 {
     Writer sw;
     writeFormatted(geometry, false, &sw);
-    string res = sw.toString();
+    std::string res = sw.toString();
     return res;
 }
 
@@ -158,7 +160,7 @@ WKTWriter::write(const Geometry* geometry, Writer* writer)
     writeFormatted(geometry, false, writer);
 }
 
-string
+std::string
 WKTWriter::writeFormatted(const Geometry* geometry)
 {
     Writer sw;
@@ -178,8 +180,9 @@ WKTWriter::writeFormatted(const Geometry* geometry, bool p_isFormatted,
 {
     CLocalizer clocale;
     this->isFormatted = p_isFormatted;
-    decimalPlaces = roundingPrecision == -1 ? geometry->getPrecisionModel()->getMaximumSignificantDigits() :
-                    roundingPrecision;
+    decimalPlaces = roundingPrecision == -1
+                    ? geometry->getPrecisionModel()->getMaximumSignificantDigits()
+                    : roundingPrecision;
     appendGeometryTaggedText(geometry, 0, writer);
 }
 
@@ -252,11 +255,11 @@ WKTWriter::appendLineStringTaggedText(const LineString* lineString, int p_level,
     appendLineStringText(lineString, p_level, false, writer);
 }
 
-/*
- * Converts a <code>LinearRing</code> to &lt;LinearRing Tagged Text&gt;
+/**
+ * Converts a `LinearRing` to \<LinearRing Tagged Text\>
  * format, then appends it to the writer.
  *
- * @param  linearRing  the <code>LinearRing</code> to process
+ * @param  linearRing  the `LinearRing` to process
  * @param  writer      the output writer to append to
  */
 void
@@ -354,25 +357,39 @@ WKTWriter::appendCoordinate(const Coordinate* coordinate,
 }
 
 /* protected */
-string
-WKTWriter::writeNumber(double d)
+std::string
+WKTWriter::writeNumber(double d) const
 {
-
-    std::stringstream ss;
-
-    if(! trim) {
+    uint32_t precision = decimalPlaces >= 0 ? static_cast<std::uint32_t>(decimalPlaces) : 0;
+    /*
+    * For a "trimmed" result, with no trailing zeros we use
+    * the ryu library.
+    */
+    if (trim) {
+        char buf[128];
+        int len = geos_d2sfixed_buffered_n(d, precision, buf);
+        buf[len] = '\0';
+        std::string s(buf);
+        return s;
+    }
+    /*
+    * For an "untrimmed" result, compatible with the old
+    * format, we continue to use std::fixed.
+    */
+    else {
+        std::stringstream ss;
         ss << std::fixed;
+        ss << std::setprecision(static_cast<int>(precision));
+        ss << d;
+        return ss.str();
     }
-    ss << std::setprecision(decimalPlaces >= 0 ? decimalPlaces : 0) << d;
-
-    return ss.str();
 }
 
 void
 WKTWriter::appendLineStringText(const LineString* lineString, int p_level,
                                 bool doIndent, Writer* writer)
 {
-    if(lineString->isEmpty()) {
+    if (lineString->isEmpty()) {
         writer->write("EMPTY");
     }
     else {
@@ -380,7 +397,7 @@ WKTWriter::appendLineStringText(const LineString* lineString, int p_level,
             indent(p_level, writer);
         }
         writer->write("(");
-        for(size_t i = 0, n = lineString->getNumPoints(); i < n; ++i) {
+        for(std::size_t i = 0, n = lineString->getNumPoints(); i < n; ++i) {
             if(i > 0) {
                 writer->write(", ");
                 if(i % 10 == 0) {
@@ -406,7 +423,7 @@ WKTWriter::appendPolygonText(const Polygon* polygon, int /*level*/,
         }
         writer->write("(");
         appendLineStringText(polygon->getExteriorRing(), level, false, writer);
-        for(size_t i = 0, n = polygon->getNumInteriorRing(); i < n; ++i) {
+        for(std::size_t i = 0, n = polygon->getNumInteriorRing(); i < n; ++i) {
             writer->write(", ");
             const LineString* ls = polygon->getInteriorRingN(i);
             appendLineStringText(ls, level + 1, true, writer);
@@ -424,15 +441,19 @@ WKTWriter::appendMultiPointText(const MultiPoint* multiPoint,
     }
     else {
         writer->write("(");
-        for(size_t i = 0, n = multiPoint->getNumGeometries();
+        for(std::size_t i = 0, n = multiPoint->getNumGeometries();
                 i < n; ++i) {
+
             if(i > 0) {
                 writer->write(", ");
             }
-
-            appendCoordinate(
-                dynamic_cast<const Point*>(multiPoint->getGeometryN(i))->getCoordinate(),
-                writer);
+            const Coordinate* coord = multiPoint->getGeometryN(i)->getCoordinate();
+            if(coord == nullptr) {
+                writer->write("EMPTY");
+            }
+            else {
+                appendCoordinate(coord, writer);
+            }
         }
         writer->write(")");
     }
@@ -449,16 +470,14 @@ WKTWriter::appendMultiLineStringText(const MultiLineString* multiLineString, int
         int level2 = p_level;
         bool doIndent = indentFirst;
         writer->write("(");
-        for(size_t i = 0, n = multiLineString->getNumGeometries();
+        for(std::size_t i = 0, n = multiLineString->getNumGeometries();
                 i < n; ++i) {
             if(i > 0) {
                 writer->write(", ");
                 level2 = p_level + 1;
                 doIndent = true;
             }
-            const LineString* ls = dynamic_cast<const LineString*>(
-                                       multiLineString->getGeometryN(i)
-                                   );
+            const LineString* ls = multiLineString->getGeometryN(i);
             appendLineStringText(ls, level2, doIndent, writer);
         }
         writer->write(")");
@@ -475,16 +494,14 @@ WKTWriter::appendMultiPolygonText(const MultiPolygon* multiPolygon, int p_level,
         int level2 = p_level;
         bool doIndent = false;
         writer->write("(");
-        for(size_t i = 0, n = multiPolygon->getNumGeometries();
+        for(std::size_t i = 0, n = multiPolygon->getNumGeometries();
                 i < n; ++i) {
             if(i > 0) {
                 writer->write(", ");
                 level2 = p_level + 1;
                 doIndent = true;
             }
-            const Polygon* p = dynamic_cast<const Polygon*>(
-                                   multiPolygon->getGeometryN(i)
-                               );
+            const Polygon* p = multiPolygon->getGeometryN(i);
             appendPolygonText(p, level2, doIndent, writer);
         }
         writer->write(")");
@@ -497,13 +514,10 @@ WKTWriter::appendGeometryCollectionText(
     int p_level,
     Writer* writer)
 {
-    if(geometryCollection->isEmpty()) {
-        writer->write("EMPTY");
-    }
-    else {
+    if(geometryCollection->getNumGeometries() > 0) {
         int level2 = p_level;
         writer->write("(");
-        for(size_t i = 0, n = geometryCollection->getNumGeometries();
+        for(std::size_t i = 0, n = geometryCollection->getNumGeometries();
                 i < n; ++i) {
             if(i > 0) {
                 writer->write(", ");
@@ -513,16 +527,19 @@ WKTWriter::appendGeometryCollectionText(
         }
         writer->write(")");
     }
+    else {
+        writer->write("EMPTY");
+    }
 }
 
 void
-WKTWriter::indent(int p_level, Writer* writer)
+WKTWriter::indent(int p_level, Writer* writer) const
 {
     if(!isFormatted || p_level <= 0) {
         return;
     }
     writer->write("\n");
-    writer->write(string(INDENT * p_level, ' '));
+    writer->write(std::string(INDENT * static_cast<std::size_t>(p_level), ' '));
 }
 
 } // namespace geos.io
