@@ -9,54 +9,22 @@
 #include <cstdarg>
 #include <cstdio>
 #include <cstdlib>
+#include <fenv.h>
 #include <memory>
 #include <math.h>
 
+#include "capi_test_utils.h"
+
 namespace tut {
 //
 // Test Group
 //
 
 // Common data used in test cases.
-struct test_capigeosdistance_data {
-    GEOSGeometry* geom1_;
-    GEOSGeometry* geom2_;
-    GEOSGeometry* geom3_;
-    GEOSWKTWriter* w_;
-
-    static void
-    notice(const char* fmt, ...)
-    {
-        std::fprintf(stdout, "NOTICE: ");
-
-        va_list ap;
-        va_start(ap, fmt);
-        std::vfprintf(stdout, fmt, ap);
-        va_end(ap);
-
-        std::fprintf(stdout, "\n");
+struct test_capigeosdistance_data : public capitest::utility {
+    test_capigeosdistance_data() {
+        GEOSWKTWriter_setTrim(wktw_, 1);
     }
-
-    test_capigeosdistance_data()
-        : geom1_(nullptr), geom2_(nullptr), geom3_(nullptr), w_(nullptr)
-    {
-        initGEOS(notice, notice);
-        w_ = GEOSWKTWriter_create();
-        GEOSWKTWriter_setTrim(w_, 1);
-    }
-
-    ~test_capigeosdistance_data()
-    {
-        GEOSGeom_destroy(geom1_);
-        GEOSGeom_destroy(geom2_);
-        GEOSGeom_destroy(geom3_);
-        GEOSWKTWriter_destroy(w_);
-        geom1_ = nullptr;
-        geom2_ = nullptr;
-        geom3_ = nullptr;
-        finishGEOS();
-    }
-
 };
 
 typedef test_group<test_capigeosdistance_data> group;
@@ -85,13 +53,13 @@ void object::test<1>
 }
 
 GEOSGeometry*
-random_polygon(double x, double y, double r, size_t num_points)
+random_polygon(double x, double y, double r, std::size_t num_points)
 {
     std::vector<double> angle(num_points);
     std::vector<double> radius(num_points);
 
 
-    for(size_t i = 0; i < num_points; i++) {
+    for(std::size_t i = 0; i < num_points; i++) {
         angle[i] = 2 * geos::MATH_PI * std::rand() / RAND_MAX;
         radius[i] = r * std::rand() / RAND_MAX;
     }
@@ -150,5 +118,57 @@ void object::test<3>
     GEOSGeom_destroy(g2);
 }
 
+// point distance does not raise floating point exception
+template<>
+template<>
+void object::test<4>
+()
+{
+    GEOSGeometry* g1 = GEOSGeomFromWKT("POINT (0 0)");
+    GEOSGeometry* g2 = GEOSGeomFromWKT("POINT (1 1)");
+
+    // clear all floating point exceptions
+    feclearexcept (FE_ALL_EXCEPT);
+
+    double d;
+    int status = GEOSDistance(g1, g2, &d);
+
+    ensure_equals(status, 1);
+    ensure_equals(d, sqrt(2));
+
+    // check for floating point overflow exceptions
+    int raised = fetestexcept(FE_OVERFLOW);
+    ensure_equals(raised & FE_OVERFLOW, 0);
+
+    GEOSGeom_destroy(g1);
+    GEOSGeom_destroy(g2);
+}
+
+// same distance between boundables should not raise floating point exception
+template<>
+template<>
+void object::test<5>
+()
+{
+    GEOSGeometry* g1 = GEOSGeomFromWKT("LINESTRING (0 0, 1 1)");
+    GEOSGeometry* g2 = GEOSGeomFromWKT("LINESTRING (2 1, 1 2)");
+
+    // clear all floating point exceptions
+    feclearexcept (FE_ALL_EXCEPT);
+
+    double d;
+    int status = GEOSDistance(g1, g2, &d);
+
+    ensure_equals(status, 1);
+    // ensure_equals(d, sqrt(2));
+
+    // check for floating point overflow exceptions
+    int raised = fetestexcept(FE_OVERFLOW);
+    ensure_equals(raised & FE_OVERFLOW, 0);
+
+    GEOSGeom_destroy(g1);
+    GEOSGeom_destroy(g2);
+}
+
 } // namespace tut
 
