@@ -7,52 +7,7 @@ namespace tut {
 //
 
 // Common data used in test cases.
-struct test_capigeosgeomsetprecision_data : public capitest::utility {
-
-    GEOSWKTWriter* wktw_ = nullptr;
-    GEOSGeometry* geom1_ = nullptr;
-    GEOSGeometry* geom2_ = nullptr;
-    GEOSGeometry* geom3_ = nullptr;
-
-    GEOSGeometry*
-    fromWKT(const char* wkt)
-    {
-        GEOSGeometry* g = GEOSGeomFromWKT(wkt);
-        ensure(g != 0);
-        return g;
-    }
-
-    std::string
-    toWKT(GEOSGeometry* g)
-    {
-        char* wkt = GEOSWKTWriter_write(wktw_, g);
-        std::string ret(wkt);
-        GEOSFree(wkt);
-        return ret;
-    }
-
-    test_capigeosgeomsetprecision_data()
-    {
-        wktw_ = GEOSWKTWriter_create();
-        GEOSWKTWriter_setTrim(wktw_, 1);
-        GEOSWKTWriter_setRoundingPrecision(wktw_, 10);
-        //GEOSWKTWriter_setOutputDimension(wktw_, 3);
-    }
-
-    ~test_capigeosgeomsetprecision_data()
-    {
-        GEOSGeom_destroy(geom1_);
-        GEOSGeom_destroy(geom2_);
-        if(geom3_)
-            GEOSGeom_destroy(geom3_);
-        if (wktw_)
-            GEOSWKTWriter_destroy(wktw_);
-
-        geom1_ = geom2_ = geom3_ = 0;
-        wktw_ = 0;
-    }
-
-};
+struct test_capigeosgeomsetprecision_data : public capitest::utility {};
 
 typedef test_group<test_capigeosgeomsetprecision_data> group;
 typedef group::object object;
@@ -163,5 +118,45 @@ void object::test<5>
     ensure_equals(toWKT(geom3_), "LINESTRING (0 0, 0 0)");
 }
 
+// Retain (or not) collapsed elements
+template<>
+template<>
+void object::test<6> ()
+{
+    geom1_ = fromWKT("LINESTRING (0 0, 0.1 0.1)");
+    geom2_ = GEOSGeom_setPrecision(geom1_, 1.0, 0);
+    ensure_geometry_equals(geom2_, "LINESTRING EMPTY");
+}
+
+// Retain (or not) collapsed elements
+template<>
+template<>
+void object::test<7> ()
+{
+    geom1_ = fromWKT("LINESTRING (0 0, 0.1 0.1)");
+    geom2_ = GEOSGeom_setPrecision(geom1_, 1.0, GEOS_PREC_NO_TOPO);
+    ensure_geometry_equals(geom2_, "LINESTRING (0 0, 0 0)");
+}
+
+// Retain (or not) collapsed elements
+template<>
+template<>
+void object::test<8> ()
+{
+    geom1_ = fromWKT("LINESTRING (0 0, 0.1 0.1)");
+    geom2_ = GEOSGeom_setPrecision(geom1_, 1.0, GEOS_PREC_KEEP_COLLAPSED);
+    ensure_geometry_equals(geom2_, "LINESTRING (0 0, 0 0)");
+}
+
+// Retain (or not) collapsed elements
+template<>
+template<>
+void object::test<9> ()
+{
+    geom1_ = fromWKT("LINESTRING (0 0, 0.1 0.1)");
+    geom2_ = GEOSGeom_setPrecision(geom1_, 1.0, GEOS_PREC_KEEP_COLLAPSED | GEOS_PREC_NO_TOPO);
+    ensure_geometry_equals(geom2_, "LINESTRING (0 0, 0 0)");
+}
+
 } // namespace tut
 
