@@ -4,11 +4,15 @@
 #include <tut/tut.hpp>
 // geos
 #include <geos_c.h>
+#include <geos/constants.h>
 // std
 #include <cstdarg>
 #include <cstdio>
 #include <cstring>
 #include <cmath>
+#include <thread>
+
+#include "capi_test_utils.h"
 
 struct INTPOINT {
     INTPOINT(int p_x, int p_y) : x(p_x), y(p_y) {}
@@ -44,24 +48,7 @@ namespace tut {
 //
 
 // Common data used in test cases.
-struct test_capistrtree_data {
-    test_capistrtree_data()
-    {
-        initGEOS(notice, notice);
-    }
-
-    static void
-    notice(const char* fmt, ...)
-    {
-        std::fprintf(stdout, "NOTICE: ");
-
-        va_list ap;
-        va_start(ap, fmt);
-        std::vfprintf(stdout, fmt, ap);
-        va_end(ap);
-
-        std::fprintf(stdout, "\n");
-    }
+struct test_capistrtree_data : public capitest::utility {
 };
 
 typedef test_group<test_capistrtree_data> group;
@@ -106,12 +93,12 @@ template<>
 void object::test<2>
 ()
 {
-    size_t ngeoms = 100;
+    std::size_t ngeoms = 100;
     std::vector<GEOSGeometry*> geoms;
     std::vector<GEOSGeometry*> queryPoints;
     GEOSSTRtree* tree = GEOSSTRtree_create(8);
 
-    for(size_t i = 0; i < ngeoms; i++) {
+    for(std::size_t i = 0; i < ngeoms; i++) {
         GEOSCoordSequence* seq = GEOSCoordSeq_create(1, 2);
         GEOSCoordSeq_setX(seq, 0, std::rand());
         GEOSCoordSeq_setY(seq, 0, std::rand());
@@ -119,18 +106,18 @@ void object::test<2>
         GEOSSTRtree_insert(tree, geoms[i], geoms[i]);
     }
 
-    for(size_t i = 0; i < ngeoms; i++) {
+    for(std::size_t i = 0; i < ngeoms; i++) {
         GEOSCoordSequence* seq = GEOSCoordSeq_create(1, 2);
         GEOSCoordSeq_setX(seq, 0, std::rand());
         GEOSCoordSeq_setY(seq, 0, std::rand());
         queryPoints.push_back(GEOSGeom_createPoint(seq));
     }
 
-    for(size_t i = 0; i < ngeoms; i++) {
+    for(std::size_t i = 0; i < ngeoms; i++) {
         const GEOSGeometry* nearest = GEOSSTRtree_nearest(tree, queryPoints[i]);
         const GEOSGeometry* nearestBruteForce = nullptr;
-        double nearestBruteForceDistance = std::numeric_limits<double>::max();
-        for(size_t j = 0; j < ngeoms; j++) {
+        double nearestBruteForceDistance = geos::DoubleInfinity;
+        for(std::size_t j = 0; j < ngeoms; j++) {
             double distance;
             GEOSDistance(queryPoints[i], geoms[j], &distance);
 
@@ -143,7 +130,7 @@ void object::test<2>
         ensure(nearest == nearestBruteForce || GEOSEquals(nearest, nearestBruteForce));
     }
 
-    for(size_t i = 0; i < ngeoms; i++) {
+    for(std::size_t i = 0; i < ngeoms; i++) {
         GEOSGeom_destroy(geoms[i]);
         GEOSGeom_destroy(queryPoints[i]);
     }
@@ -274,7 +261,7 @@ void object::test<8>
 
     typedef std::vector<int*> IList;
     IList items;
-    ensure_equals(items.size(), 0);
+    ensure_equals(items.size(), 0U);
     GEOSSTRtree_query(
         tree,
         q,
@@ -284,7 +271,7 @@ void object::test<8>
         },
         &items);
 
-    ensure_equals(items.size(), 1);
+    ensure_equals(items.size(), 1U);
 
     ensure_equals(*(items[0]), payload);
 
@@ -309,7 +296,7 @@ void object::test<9>
 
     typedef std::vector<int*> IList;
     IList items;
-    ensure_equals(items.size(), 0);
+    ensure_equals(items.size(), 0U);
     GEOSSTRtree_query(
         tree,
         q,
@@ -319,7 +306,7 @@ void object::test<9>
         },
         &items);
 
-    ensure_equals(items.size(), 1);
+    ensure_equals(items.size(), 1U);
 
     ensure_equals(items[0], (void*)0);
 
@@ -328,7 +315,72 @@ void object::test<9>
     GEOSSTRtree_destroy(tree);
 }
 
+// Test that envelopes of inserted objects are owned by the tree.
+template<>
+template<>
+void object::test<10>()
+{
+    GEOSSTRtree* tree = GEOSSTRtree_create(10);
 
+    std::vector<size_t> ids(10);
+    for (size_t i = 0; i < 10; i++) {
+        ids[i] = i;
+        GEOSGeometry* point = GEOSGeom_createPointFromXY((double) i, (double) i);
+        GEOSSTRtree_insert(tree, point, &ids[i]);
+        GEOSGeom_destroy(point);
+    }
+
+    GEOSGeometry* queryPoint = GEOSGeom_createPointFromXY(3.0, 3.0);
+    size_t hitVal = 0;
+    GEOSSTRtree_query(tree, queryPoint, [](void* item, void* data) {
+        *((size_t*) data) = *((size_t*) item);
+    }, &hitVal);
+    GEOSGeom_destroy(queryPoint);
+    GEOSSTRtree_destroy(tree);
+
+    ensure_equals(hitVal, 3u);
+}
+
+// queries are thread-safe
+template<>
+template<>
+void object::test<11>()
+{
+    GEOSSTRtree* tree = GEOSSTRtree_create(10);
+
+    std::vector<GEOSGeometry*> geoms;
+    for (size_t i = 0; i < 100; i++) {
+        geoms.push_back(GEOSGeom_createPointFromXY((double) i, (double) i));
+    }
+    for (const auto& geom : geoms) {
+        GEOSSTRtree_insert(tree, geom, geom);
+    }
+
+    auto query = [tree](const GEOSGeometry* env) {
+        GEOSSTRtree_query(tree, env, [](void* g, void* userdata) {
+            (void) g;
+            (void) userdata;
+        }, nullptr);
+    };
+
+    GEOSGeometry* q1 = GEOSBuffer(geoms[10], 10, 48);
+    GEOSGeometry* q2 = GEOSBuffer(geoms[40], 10, 48);
+
+    std::thread t1(query, q1);
+    std::thread t2(query, q2);
+
+    t1.join();
+    t2.join();
+
+    GEOSGeom_destroy(q1);
+    GEOSGeom_destroy(q2);
+
+    for (auto& geom : geoms) {
+        GEOSGeom_destroy(geom);
+    }
+
+    GEOSSTRtree_destroy(tree);
+}
 
 
 } // namespace tut
